# Copyright 2022 The KerasCV Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keras_cv.src.backend import keras
from keras_cv.src.backend import ops
from keras_cv.src.layers import SpatialPyramidPooling
from keras_cv.src.tests.test_case import TestCase


class SpatialPyramidPoolingTest(TestCase):
    def test_return_type_and_shape(self):
        layer = SpatialPyramidPooling(dilation_rates=[6, 12, 18])
        c4 = ops.ones([2, 16, 16, 3])

        inputs = c4
        output = layer(inputs, training=True)
        self.assertEquals(output.shape, (2, 16, 16, 256))

    def test_with_keras_tensor(self):
        layer = SpatialPyramidPooling(dilation_rates=[6, 12, 18])
        c4 = keras.layers.Input([16, 16, 3])

        inputs = c4
        output = layer(inputs, training=True)
        self.assertEquals(output.shape, (None, 16, 16, 256))

